##
# File:    SessionManager.py
# Date:    14-Dec-2009
#
# Updates:
# 20-Apr-2010 jdw Ported to module seqmodule.
# 05-Aug-2010 jdw Ported to module ccmodule
#                 Replace deprecated sha module with hashlib
# 21-Sep-2010 jdw Remove 'sessions' from internal path used by this module.
#                 topPath now points to the directory containing the hash directory.
# 20-Feb-2013 jdw Application neutral version moved to utils/rcsb
##
"""
Provides containment and access for session information.  Methods
are provided to create temporary directories to preserve session files.

"""
import sys
import hashlib
import time
import os.path
import shutil

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


class SessionManager(object):
    """
    Utilities for session directory maintenance.

    """

    def __init__(self, topPath=".", verbose=False):
        """
        Organization of session directory is --
        <topPath>/<sha-hash>/<session_files>

        Parameters:
        :topPath: is the path to the directory containing the hash-id sub-directory.


        """
        self.__verbose = verbose
        self.__topSessionPath = topPath
        self.__uid = None

    def __str__(self):
        return "\n+SessionManager() Session top path: %s\nUnique identifier: %s\nSession path: %s\n" % (self.__topSessionPath, self.__uid, self.getPath())

    def __repr__(self):
        return self.__str__()

    def setId(self, uid):
        self.__uid = uid

    def getId(self):
        return self.__uid

    def assignId(self):
        # Need to convert to str (python2)/bytes (python3)
        tmp = repr(time.time()).encode("utf-8")
        self.__uid = hashlib.sha1(tmp).hexdigest()
        return self.__uid

    def getPath(self):
        try:
            pth = os.path.join(self.__topSessionPath, "sessions", self.__uid)
            if self.__verbose:
                sys.stderr.write("+SessionManager.getPath() path %s\n" % pth)
            if os.access(pth, os.F_OK):
                return pth
            else:
                return None
        except:  # noqa: E722 pylint: disable=bare-except
            return None

    def getTopPath(self):
        return self.__topSessionPath

    def getRelativePath(self):
        pth = None
        try:
            pth = os.path.join("/sessions", self.__uid)

        except:  # noqa: E722 pylint: disable=bare-except
            pass
        return pth

    def makeSessionPath(self):
        """If the path to the current session directory does not exist
        create it and return the session path.
        """
        try:
            pth = os.path.join(self.__topSessionPath, "sessions", self.__uid)
            if not os.access(pth, os.F_OK):
                os.makedirs(pth)
            return pth
        except:  # noqa: E722 pylint: disable=bare-except
            return None

    def remakeSessionPath(self):
        try:
            pth = os.path.join(self.__topSessionPath, "sessions", self.__uid)
            if os.access(pth, os.F_OK):
                shutil.rmtree(pth, True)
            os.makedirs(pth)
            return pth
        except:  # noqa: E722 pylint: disable=bare-except
            return None



##
# File:  wwPDBWebOb.py
# Date:  11-Mar-2021 E. Peisach
#
# Update:
##
"""
Wrapper for WebOB to cleanup file descriptors when going out of scope.
"""
__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "ezra.peisach@rcsb.org"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

from webob import Request as webob_Request
from webob import Response as webob_Response


class WwPdbRequest(object):
    def __init__(self, environ, charset=None, unicode_errors=None, decode_param_names=None, **kw):
        self.req = webob_Request(environ, charset=charset, unicode_errors=unicode_errors, decode_param_names=decode_param_names, **kw)

    def __enter__(self):
        return self.req

    def __exit__(self, type, value, traceback):  # pylint: disable=redefined-builtin
        for name, fs in self.req.params.items():
            if (fs is None) or (isinstance(fs, str)) or (isinstance(fs, bytes)):
                continue
            if not hasattr(fs, "filename") or fs.filename == name:
                continue
            try:
                fs.file.close()
            except:  # noqa: E722 pylint: disable=bare-except
                pass
        # This is because WebOB will create a temporary file, and rely on garbage collection to close files. Python3, this is not happening when go out scope
        # For new webob, there might be subsequent changes to code - so protect
        if self.req.is_body_seekable:
            if self.req.body_file_raw and hasattr(self.req.body_file_raw, "close"):
                try:
                    self.req.body_file_raw.close()
                except:  # noqa: E722 pylint: disable=bare-except
                    pass


class WwPdbResponse(webob_Response):
    """Pass through for webob Reponse class in case we need to add a wrapper later"""



##
# File:    UtilDataStore.py
# Date:    6-July-2012
#
# Updates:
#            7-July-2012 jdw make store path depend on entryId
#           07-Mar-2013  jdw make generic and move main project utils/rcsb
#           12-Jul-2013  jdw add append/extend methods for assigning list values
#            2-Mar-2014  jdw add  updateDict(self,key,subKey,value)
##
"""
Provide a storage interface for miscellaneous key,value data.

"""
import sys
import os.path

try:
    import cPickle as pickle
except ImportError:
    import pickle

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


class UtilDataStore(object):
    """Provide a storage interface for miscellaneous key,value data."""

    def __init__(self, reqObj, prefix=None, verbose=False, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = True
        self.__lfh = log
        self.__reqObj = reqObj
        if prefix is not None:
            self.__filePrefix = prefix
        else:
            self.__filePrefix = "general"
        self.__filePath = None
        self.__D = {}
        self.__setup()

    def __setup(self):
        self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        self.__sObj = self.__reqObj.getSessionObj()
        self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        #
        # self.__pickleProtocol = pickle.HIGHEST_PROTOCOL
        self.__pickleProtocol = 0
        try:
            self.__filePath = os.path.join(self.__sessionPath, self.__filePrefix + "-util-session.pic")
            if self.__verbose:
                self.__lfh.write("\n+UtilDataStore.__setup() - data store path %s\n" % self.__filePath)
            self.deserialize()
        except Exception as e:
            if self.__debug:
                self.__lfh.write(
                    "\n+UtilDataStore.__setup() - Failed to open data store for session id %s data store prefix %s path %s err %s\n"
                    % (self.__sessionId, self.__filePrefix, self.__filePath, str(e))
                )

    def reset(self):
        self.__D = {}

    def getFilePath(self):
        return self.__filePath

    def serialize(self):
        try:
            fb = open(self.__filePath, "wb")
            pickle.dump(self.__D, fb, self.__pickleProtocol)
            fb.close()
        except:  # noqa: E722 pylint: disable=bare-except
            pass

    def deserialize(self):
        try:
            fb = open(self.__filePath, "rb")
            self.__D = pickle.load(fb)
            fb.close()
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def get(self, key):
        try:
            return self.__D[key]
        except:  # noqa: E722 pylint: disable=bare-except
            return ""

    def set(self, key, value):
        try:
            self.__D[key] = value
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def append(self, key, value):
        try:
            if key not in self.__D:
                self.__D[key] = []
            self.__D[key].append(value)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def extend(self, key, valueList):
        try:
            if key not in self.__D:
                self.__D[key] = []
            self.__D[key].extend(valueList)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def updateDict(self, key, subKey, value):
        try:
            if key not in self.__D:
                self.__D[key] = {}
            self.__D[key][subKey] = value
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def getDictionary(self):
        return self.__D



##
# File:    WebRequest.py
# Date:    18-Jan-2010  J. Westbrook
#
# Updated:
# 20-Apr-2010 Ported to seqmodule package
# 25-Jul-2010 Ported to ccmodule package
# 24-Aug-2010 Add dictionary update for content request object.
#
# 26-Feb-2012 jdw  add support location redirects in response object-
# 20-Feb-2013 jdw  This application neutral version moved to utils/rcsb.
#                  Make dictionary in ResponseContent inheritable.
# 06-Mar-2013 jdw  add setDictionary() method
# 07-Mar-2013 jdw  add optional maximum length parameter to response object dump() method.
# 22-Mar-2013 jdw  incorporate methods for handling binary content, jsonp, and datafile
#                  download data in response content class.
# 22-Mar-2013 jdw  restore method - getValueOrDefault(self,myKey,default='')
# 11-Oct-2013 zf   add getDictionary() method
# 22-Dec-2013 jdw  Add HTML template processing method - setHtmlFromTemplate()
# 27-Feb-2014 jdw  Add setReturnFormat() method -
# 13-Jul-2014 jdw  Adjust formating in print methods
##
"""
WebRequest provides containers and accessors for managing request parameter information.

This is an application neutral version shared by UI modules --

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys
import os
import traceback
import gzip
import mimetypes

try:
    from json import loads, dumps
except:  # noqa: E722 pylint: disable=bare-except
    from simplejson import loads, dumps

from wwpdb.utils.session.SessionManager import SessionManager
from datetime import datetime


def json_serializer_helper(obj):
    """Helper function to handle = objects not serializable by default json code

    Current -  handling datetime objects and falling back to str()
    """

    if isinstance(obj, datetime):
        serial = obj.isoformat()
        return serial
    else:
        try:
            serial = str(obj)
            return serial
        except Exception as e:
            raise TypeError("Could not convert object to string %r %r " % (obj, str(e)))
    #
    raise TypeError("Type not serializable %r " % obj)


class WebRequest(object):

    """Base container and accessors for input and output parameters and control information."""

    def __init__(self, paramDict=None, verbose=False):
        if paramDict is None:
            paramDict = {}
        self.__verbose = verbose
        #
        #  Input and storage model is dictionary of lists (e.g. dict[myKey] = [,,,])
        #  Single values are stored in the leading element of the list (e.g. dict[myKey][0])
        #
        self.__dict = paramDict
        self.__debug = False

    def __str__(self):
        try:
            sL = []
            sL.append("\n+WebRequest.printIt() WebRequest dictionary contents:\n")
            for k, vL in self.__dict.items():
                sL.append("  - Key: %-35s  value(s): %r\n" % (k, vL))
            sL.append("   --------------------------------------------\n")
            return "".join(sL)
        except:  # noqa: E722 pylint: disable=bare-except
            return ""

    def __repr__(self):
        return self.__str__()

    def printIt(self, ofh=sys.stdout):
        try:
            ofh.write("\n +WebRequest.printIt() WebRequest dictionary contents:\n")

            for k in sorted(self.__dict.keys()):
                vL = self.__dict[k]
                ofh.write("  - Key: %-35s  value(s): %r\n" % (k, vL))
            ofh.write("   --------------------------------------------\n")
        except:  # noqa: E722 pylint: disable=bare-except
            pass

    def dump(self, format="text"):  # pylint: disable=redefined-builtin
        oL = []
        try:
            if format == "html":
                oL.append("<pre>\n")
            oL.append("\n+nWebRequest.dump() Request Dictionary Contents:\n")
            for k, vL in self.__dict.items():
                oL.append("  - Key: %-35s  value(s): %r\n" % (k, vL))
            oL.append("   --------------------------------------------\n")
            if format == "html":
                oL.append("</pre>\n")
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        return oL

    def getJSON(self):
        return dumps(self.__dict)

    def setJSON(self, JSONString):
        self.__dict = loads(JSONString)

    def getValue(self, myKey):
        return self._getStringValue(myKey)

    def getValueOrDefault(self, myKey, default=""):
        if not self.exists(myKey):
            return default
        v = self._getStringValue(myKey)
        if len(v) < 1:
            return default
        return v

    def getValueList(self, myKey):
        return self._getStringList(myKey)

    def getRawValue(self, myKey):
        return self._getRawValue(myKey)

    def getDictionary(self):
        return self.__dict

    #

    def setValue(self, myKey, aValue):
        self.__dict[myKey] = [aValue]

    def setValueList(self, myKey, valueList):
        self.__dict[myKey] = valueList

    def setDictionary(self, myDict, overWrite=False):
        for k, v in myDict.items():
            if overWrite or (not self.exists(k)):
                self.setValue(k, v)

    def exists(self, myKey):
        try:
            return myKey in self.__dict
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    #
    def _getRawValue(self, myKey):
        try:
            return self.__dict[myKey][0]
        except:  # noqa: E722 pylint: disable=bare-except
            return None

    def _getStringValue(self, myKey):
        try:
            return str(self.__dict[myKey][0]).strip()
        except:  # noqa: E722 pylint: disable=bare-except
            return ""

    def _getIntegerValue(self, myKey):
        try:
            return int(self.__dict[myKey][0])
        except:  # noqa: E722 pylint: disable=bare-except
            return None

    def _getDoubleValue(self, myKey):
        try:
            return float(self.__dict[myKey][0])
        except:  # noqa: E722 pylint: disable=bare-except
            return None

    def _getStringList(self, myKey):
        try:
            return self.__dict[myKey]
        except:  # noqa: E722 pylint: disable=bare-except
            return []


class InputRequest(WebRequest):
    def __init__(self, paramDict, verbose=False, log=sys.stderr):
        super(InputRequest, self).__init__(paramDict, verbose)
        self.__verbose = verbose
        self.__lfh = log
        self.__returnFormatDefault = ""

    def setDefaultReturnFormat(self, return_format="html"):
        self.__returnFormatDefault = return_format
        if not self.exists("return_format"):
            self.setValue("return_format", self.__returnFormatDefault)

    def getRequestPath(self):
        return self._getStringValue("request_path")

    def getReturnFormat(self):
        if not self.exists("return_format"):
            self.setValue("return_format", self.__returnFormatDefault)
        return self._getStringValue("return_format")

    def setReturnFormat(self, return_format="html"):
        return self.setValue("return_format", return_format)

    def getSessionId(self):
        return self._getStringValue("sessionid")

    def getSessionPath(self):
        return os.path.join(self._getStringValue("TopSessionPath"), "sessions")

    def getTopSessionPath(self):
        return self._getStringValue("TopSessionPath")

    def getSemaphore(self):
        return self._getStringValue("semaphore")

    def getSessionObj(self):
        if self.exists("TopSessionPath"):
            sObj = SessionManager(topPath=self._getStringValue("TopSessionPath"))
        else:
            sObj = SessionManager()
        sObj.setId(uid=self._getStringValue("sessionid"))
        return sObj

    def newSessionObj(self, forceNew=False):
        if self.exists("TopSessionPath"):
            sObj = SessionManager(topPath=self._getStringValue("TopSessionPath"))
        else:
            sObj = SessionManager()

        sessionId = self._getStringValue("sessionid")

        if forceNew:
            sObj.assignId()
            sObj.makeSessionPath()
            self.setValue("sessionid", sObj.getId())
        elif len(sessionId) > 0:
            sObj.setId(sessionId)
            sObj.makeSessionPath()
        else:
            sObj.assignId()
            sObj.makeSessionPath()
            self.setValue("sessionid", sObj.getId())

        return sObj


#


class ResponseContent(object):
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        Manage content items to be transfered as part of the application response.

        """
        self.__verbose = verbose
        self.__lfh = log
        self.__reqObj = reqObj
        #
        self._cD = {}
        self.__debug = False
        self.__returnFormat = ""
        self.__setup()

    def __setup(self):
        """Default response content is set here."""
        self._cD["htmllinkcontent"] = ""
        self._cD["htmlcontent"] = ""
        self._cD["textcontent"] = ""
        self._cD["location"] = ""
        self._cD["datatype"] = None
        self._cD["encodingtype"] = None
        self._cD["datafilename"] = None
        self._cD["disposition"] = None

        #
        self._cD["datacontent"] = None
        self._cD["errorflag"] = False
        self._cD["statustext"] = ""
        # legacy setting --
        self._cD["errortext"] = ""
        if self.__reqObj is not None:
            self._cD["sessionid"] = self.__reqObj.getSessionId()
            self._cD["semaphore"] = self.__reqObj.getSemaphore()
            # Default comes from the input request object
            self.__returnFormat = self.__reqObj.getReturnFormat()
        else:
            self._cD["sessionid"] = ""
            self._cD["semaphore"] = ""
        #

    def setData(self, dataObj=None):
        self._cD["datacontent"] = dataObj

    def set(self, key, val, asJson=False):
        if asJson:
            try:
                self._cD[key] = dumps(val, default=json_serializer_helper)
            except Exception as e:
                self.__lfh.write("+set() failed %r\n" % str(e))
                traceback.print_exc(file=self.__lfh)
        else:
            self._cD[key] = val

    def setHtmlList(self, htmlList=None):
        if htmlList is None:
            htmlList = []
        self._cD["htmlcontent"] = "\n".join(htmlList)

    def appendHtmlList(self, htmlList=None):
        if htmlList is None:
            htmlList = []
        if len(self._cD["htmlcontent"]) > 0:
            self._cD["htmlcontent"] = "%s\n%s" % (self._cD["htmlcontent"], "\n".join(htmlList))
        else:
            self._cD["htmlcontent"] = "\n".join(htmlList)

    def setHtmlText(self, htmlText=""):
        self._cD["htmlcontent"] = htmlText

    def setHtmlTextFromTemplate(self, templateFilePath, webIncludePath, parameterDict=None, insertContext=False):
        pD = parameterDict if parameterDict is not None else {}
        self._cD["htmlcontent"] = self.__processTemplate(templateFilePath=templateFilePath, webIncludePath=webIncludePath, parameterDict=pD, insertContext=insertContext)

    def setHtmlLinkText(self, htmlText=""):
        self._cD["htmllinkcontent"] = htmlText

    def setText(self, text=""):
        self._cD["textcontent"] = text

    def setLocation(self, url=""):
        self._cD["location"] = url

    def addDictionaryItems(self, cD=None):
        if cD is None:
            cD = {}
        for k, v in cD.items():
            self._cD[k] = v

    def setTextFile(self, filePath):
        try:
            if os.path.exists(filePath):
                with open(filePath, "r") as fin:
                    self._cD["textcontent"] = fin.read()
        except Exception as e:
            self.__lfh.write("+setTextFile() File read failed %s %s\n" % (filePath, str(e)))
            traceback.print_exc(file=self.__lfh)

    def setTextFileO(self, filePath):
        with open(filePath, "r") as fin:
            self._cD["textcontent"] = fin.read()

    def getMimetypeAndEncoding(self, filename):
        mtype, encoding = mimetypes.guess_type(filename)
        # We'll ignore encoding, even though we shouldn't really
        if mtype is None:
            if filename.find(".cif.V") > 0:
                ret = ("text/plain", None)
            else:
                ret = ("application/octet-stream", None)
        else:
            ret = (mtype, encoding)
        return ret

    def setBinaryFile(self, filePath, attachmentFlag=False, serveCompressed=True):
        try:
            if os.path.exists(filePath):
                _dir, fn = os.path.split(filePath)
                if not serveCompressed and fn.endswith(".gz"):
                    with gzip.open(filePath, "rb") as fin:
                        self._cD["datacontent"] = fin.read()
                    self._cD["datafileName"] = fn[:-3]
                    contentType, encodingType = self.getMimetypeAndEncoding(filePath[:-3])
                else:
                    with open(filePath, "rb") as fin:
                        self._cD["datacontent"] = fin.read()
                    self._cD["datafileName"] = fn
                    contentType, encodingType = self.getMimetypeAndEncoding(filePath)
                #
                self._cD["datatype"] = contentType
                self._cD["encodingtype"] = encodingType
                if attachmentFlag:
                    self._cD["disposition"] = "attachment"
                else:
                    self._cD["disposition"] = "inline"
                    #
                    # strip compression file extension if disposition=inline.
                    if fn.endswith(".gz"):
                        self._cD["datafileName"] = fn[:-3]

                if self.__verbose:
                    self.__lfh.write("+ResponseContent.setBinaryFile() Serving %s as %s encoding %s att flag %r\n" % (filePath, contentType, encodingType, attachmentFlag))
        except Exception as e:
            self.__lfh.write("ResponseContent.setBinaryFile() File read failed %s error: %r\n" % (filePath, str(e)))
            traceback.print_exc(file=self.__lfh)

    def wrapFileAsJsonp(self, filePath, callBack=None):
        try:
            if os.path.exists(filePath):
                _dir, fn = os.path.split(filePath)
                (_rn, ext) = os.path.splitext(fn)
                #
                dd = {}
                with open(filePath, "r") as fin:
                    dd["data"] = fin.read()
                if ext.lower() != ".json":
                    self._cD["datacontent"] = callBack + "(" + dumps(dd) + ");"
                else:
                    self._cD["datacontent"] = callBack + "(" + dd["data"] + ");"
                #
                self._cD["datafileName"] = fn
                contentType = "application/x-javascript"
                encodingType = None
                #
                self._cD["datatype"] = contentType
                self._cD["encodingtype"] = encodingType
                self._cD["disposition"] = "inline"
                #
                if self.__debug:  # pragma: no cover
                    self.__lfh.write("+ResponseContent.wrapFileAsJsonp() Serving %s as %s\n" % (filePath, self._cD["datacontent"]))
        except Exception as e:
            self.__lfh.write("ResponseContent.wrapFileAsJsonp() File read failed %s err=%r\n" % (filePath, str(e)))
            traceback.print_exc(file=self.__lfh)

    def setStatus(self, statusMsg="", semaphore=""):
        self._cD["errorflag"] = False
        self._cD["statustext"] = statusMsg
        self._cD["semaphore"] = semaphore

    def isError(self):
        return self._cD["errorflag"]

    def setError(self, errMsg="", semaphore=""):
        self._cD["errorflag"] = True
        self._cD["statustext"] = errMsg
        # legacy setting -
        self._cD["errortext"] = errMsg
        self._cD["semaphore"] = semaphore

    def setStatusCode(self, aCode):
        self._cD["statuscode"] = aCode

    def setHtmlContentPath(self, aPath):
        self._cD["htmlcontentpath"] = aPath

    def dump(self, maxLength=130):
        retL = []
        retL.append("\n +ResponseContent.dump() - response content object\n")
        for k, v in self._cD.items():
            if v is None:
                continue
            elif isinstance(v, dict):
                retL.append("  - key = %-35s " % k)
                retL.append("  - dict : %s\n" % list(v.items()))
            elif v is not None and len(str(v).strip()) > 0:
                retL.append("  - key = %-35s " % k)
                retL.append(" value(1-%d): %s\n" % (maxLength, str(v)[:maxLength]))
        return retL

    def setReturnFormat(self, format):  # pylint: disable=redefined-builtin
        if format in ["html", "text", "json", "jsonText", "jsonData", "location", "binary", "jsonp"]:
            self.__returnFormat = format
            return True
        else:
            return False

    def get(self):
        """Repackage the response for Apache according to the input return_format='html|json|text|...'"""
        rD = {}
        if self.__returnFormat == "html":
            if self._cD["errorflag"] is False:
                rD = self.__initHtmlResponse(self._cD["htmlcontent"])
            else:
                rD = self.__initHtmlResponse(self._cD["statustext"])
        elif self.__returnFormat == "text":
            if self._cD["errorflag"] is False:
                rD = self.__initTextResponse(self._cD["textcontent"])
            else:
                rD = self.__initHtmlResponse(self._cD["statustext"])
        elif self.__returnFormat == "json":
            rD = self.__initJsonResponse(self._cD)
        elif self.__returnFormat == "jsonText":
            rD = self.__initJsonResponseInTextArea(self._cD)
        elif self.__returnFormat == "jsonData":
            rD = self.__initJsonResponse(self._cD["datacontent"])
        elif self.__returnFormat == "location":
            rD = self.__initLocationResponse(self._cD["location"])
        elif self.__returnFormat == "binary":
            rD = self.__initBinaryResponse(self._cD)
        elif self.__returnFormat == "jsonp":
            rD = self.__initJsonpResponse(self._cD)
        else:
            pass
        #
        return rD

    def __initLocationResponse(self, url):
        rspDict = {}
        rspDict["CONTENT_TYPE"] = "location"
        rspDict["RETURN_STRING"] = url
        return rspDict

    def __initBinaryResponse(self, myD=None):
        if myD is None:
            myD = {}
        rspDict = {}
        rspDict["CONTENT_TYPE"] = myD["datatype"]
        rspDict["RETURN_STRING"] = myD["datacontent"]
        try:
            rspDict["ENCODING"] = myD["encodingtype"]
            if myD["disposition"] is not None:
                rspDict["DISPOSITION"] = "%s; filename=%s" % (myD["disposition"], myD["datafileName"])
        except:  # noqa: E722 pylint: disable=bare-except
            pass
        return rspDict

    def __initJsonResponse(self, myD=None):
        if myD is None:
            myD = {}
        rspDict = {}
        rspDict["CONTENT_TYPE"] = "application/json"
        rspDict["RETURN_STRING"] = dumps(myD)
        return rspDict

    def __initJsonpResponse(self, myD=None):
        if myD is None:
            myD = {}
        rspDict = {}
        rspDict["CONTENT_TYPE"] = myD["datatype"]
        rspDict["RETURN_STRING"] = myD["datacontent"]
        return rspDict

    def __initJsonResponseInTextArea(self, myD=None):
        if myD is None:
            myD = {}
        rspDict = {}
        rspDict["CONTENT_TYPE"] = "text/html"
        # rspDict['CONTENT_TYPE']  = 'text/plain'
        rspDict["RETURN_STRING"] = "<textarea>" + dumps(myD) + "</textarea>"
        return rspDict

    def __initHtmlResponse(self, myHtml=""):
        rspDict = {}
        rspDict["CONTENT_TYPE"] = "text/html"
        rspDict["RETURN_STRING"] = myHtml
        return rspDict

    def __initTextResponse(self, myText=""):
        rspDict = {}
        rspDict["CONTENT_TYPE"] = "text/plain"
        rspDict["RETURN_STRING"] = myText
        return rspDict

    def __processTemplate(self, templateFilePath="./alignment_template.html", webIncludePath=".", parameterDict=None, insertContext=False):
        """Read the input HTML template data file and perform the key/value substitutions in the
        input parameter dictionary.

        if insertContext is set then paramDict is injected as a json object if <!--insert application_context=""-->

        Template HTML file path -  (e.g. /../../htdocs/<appName>/template.html)
        webTopPath = file system path for web includes files  (eg. /../../htdocs) which will
                     be prepended to embedded include path in the HTML template document
        """
        if parameterDict is None:
            parameterDict = {}
        try:
            ifh = open(templateFilePath, "r")
            sL = []
            for line in ifh.readlines():
                if str(line).strip().startswith("<!--#include") or (insertContext and str(line).strip().startswith("<!--#insert")):
                    fields = str(line).split('"')
                    tpth = os.path.join(webIncludePath, fields[1][1:])
                    try:
                        tfh = open(tpth, "r")
                        sL.append(tfh.read())
                        tfh.close()
                    except Exception as e:
                        if self.__verbose:
                            self.__lfh.write("+WebRequest.__processTemplate() failed to include %s fields=%r err=%r\n" % (tpth, fields, str(e)))
                else:
                    sL.append(line)
            ifh.close()
            sIn = "".join(sL)
            return sIn % parameterDict
        except Exception as e:
            if self.__verbose:
                self.__lfh.write("+WebRequest.__processTemplate() failed for %s %r\n" % (templateFilePath, str(e)))
                traceback.print_exc(file=self.__lfh)

        return ""


if __name__ == "__main__":  # pragma: no cover
    rC = ResponseContent()



##
# File:  WebUploadUtils.py
# Date:  28-Feb-2013
#
# Updates:
#  28-Feb-2013   jdw imported common functions from WebApp(*).py  modules.
#  03-Mar-2013   jdw catch unicode type in empty file request.
#  28-Feb-2014   jdw add rename and file extension methods
#   2-Apr-2014   jdw add version flag to getFileExtension(fileName,ignoreVersion=False)
#  14-Sep-2014   jdw add getUploadFileName():
##
"""
Utilities to manage  web application upload tasks.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.09"

import sys
import ntpath
import os
import types
import traceback
import shutil


class WebUploadUtils(object):
    """
    This class encapsulates all of the web application upload tasks.

    """

    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):

        self.__reqObj = reqObj
        self.__verbose = verbose
        self.__lfh = log
        #
        self.__debug = False
        #
        self.__sessionObj = self.__reqObj.getSessionObj()
        self.__sessionPath = self.__sessionObj.getPath()
        #
        if self.__verbose:
            self.__lfh.write("+WebUploadUtils.__setup() - session id   %s\n" % (self.__sessionObj.getId()))
            self.__lfh.write("+WebUploadUtils.__setup() - session path %s\n" % (self.__sessionPath))

    def isFileUpload(self, fileTag="file"):
        """Generic check for the existence of request paramenter "fileTag="."""
        # Gracefully exit if no file is provide in the request object -
        fs = self.__reqObj.getRawValue(fileTag)

        if self.__debug:
            self.__lfh.write("+WebUploadUtils.isFileUpLoad() fs  %r\n" % fs)

        if sys.version_info[0] < 3:
            if (fs is None) or (isinstance(fs, types.StringType)) or (isinstance(fs, types.UnicodeType)):  # pylint: disable=no-member
                return False
        else:
            if (fs is None) or (isinstance(fs, str)) or (isinstance(fs, bytes)):
                return False
        return True

    def getUploadFileName(self, fileTag="file"):
        """Get the user supplied name of for the uploaded file -"""
        #
        if self.__verbose:
            self.__lfh.write("+WebUploadUtils.getUploadFileName() - operation started\n")
        #
        try:
            fs = self.__reqObj.getRawValue(fileTag)
            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.getUploadFileName() - upload file descriptor fs =     %r\n" % fs)
                self.__lfh.write("+WebUploadUtils.getUploadFileName() - upload file descriptor fs =     %s\n" % fs)
            formRequestFileName = str(fs.filename).strip()

            #
            if formRequestFileName.find("\\") != -1:
                uploadInputFileName = ntpath.basename(formRequestFileName)
            else:
                uploadInputFileName = os.path.basename(formRequestFileName)

            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.getUploadFileName() Uploaded file name %s\n" % str(uploadInputFileName))
            #
            return uploadInputFileName
        except Exception as e:
            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.getUploadFileName() processing failed %s\n" % str(e))
                traceback.print_exc(file=self.__lfh)
        return None

    def copyToSession(self, fileTag="file", sessionFileName=None, uncompress=True):
        """Copy uploaded file identified form element name 'fileTag' to the current session directory.

        File is copied to user uploaded file or to the sessionFileName if this is provided.
        """
        #
        if self.__verbose:
            self.__lfh.write("+WebUploadUtils.copyToSession() - operation started\n")

        #
        try:
            fs = self.__reqObj.getRawValue(fileTag)
            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.copyToSession() - upload file descriptor fs =     %r\n" % fs)
            # formRequestFileName = str(fs.filename).strip().lower()
            formRequestFileName = str(fs.filename).strip()

            #
            if formRequestFileName.find("\\") != -1:
                uploadInputFileName = ntpath.basename(formRequestFileName)
            else:
                uploadInputFileName = os.path.basename(formRequestFileName)

            #
            # Copy uploaded file in session directory
            #
            if sessionFileName is not None:
                sessionInputFileName = sessionFileName
            else:
                sessionInputFileName = uploadInputFileName

            sessionInputFilePath = os.path.join(self.__sessionPath, sessionInputFileName)

            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.copyToSession() - user request file name     %s\n" % fs.filename)
                self.__lfh.write("+WebUploadUtils.copyToSession() - upload input file name     %s\n" % uploadInputFileName)
                self.__lfh.write("+WebUploadUtils.copyToSession() - session target file path   %s\n" % sessionInputFilePath)
                self.__lfh.write("+WebUploadUtils.copyToSession() - session target file name   %s\n" % sessionInputFileName)
            #
            ofh = open(sessionInputFilePath, "wb")
            ofh.write(fs.file.read())
            ofh.close()
            #
            if uncompress and sessionInputFilePath.endswith(".gz"):
                if self.__verbose:
                    self.__lfh.write("+WebUploadUtils.copyToSession() uncompressing file %s\n" % str(sessionInputFilePath))
                self.__copyGzip(sessionInputFilePath, sessionInputFilePath[:-3])
                sessionInputFileName = sessionInputFileName[:-3]

            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.copyToSession() Uploaded file %s\n" % str(sessionInputFileName))
            #
            return sessionInputFileName
        except:  # noqa: E722 pylint: disable=bare-except
            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.copyToSession() File upload processing failed\n")
                traceback.print_exc(file=self.__lfh)
            return None

    def renameSessionFile(self, srcFileName, dstFileName):
        try:
            if srcFileName != dstFileName:
                srcPath = os.path.join(self.__sessionPath, srcFileName)
                dstPath = os.path.join(self.__sessionPath, dstFileName)
                shutil.copyfile(srcPath, dstPath)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def getFileExtension(self, fileName, ignoreVersion=False):
        """Return the file extension (basename.ext).

        If the input file contains no '.' then None is returned.

        if ignoreVersion=True then any trailing version details are
           discarded before extracting the file extension -
        """
        fExt = None
        if fileName is None or len(fileName) < 1:
            return fExt

        try:
            fL = str(fileName).split(".")
            if len(fL) < 2:
                return fExt

            if ignoreVersion and len(fL) > 2:
                tExt = fL[-1]
                if (tExt.startswith("V") or tExt.startswith("v")) and tExt[1:].isdigit():
                    fExt = fL[-2]
                else:
                    fExt = tExt
            else:
                if len(fL) > 1:
                    fExt = fL[-1]
        except:  # noqa: E722 pylint: disable=bare-except
            pass

        return fExt

    def perceiveIdentifier(self, fileName):
        """Return the file identifier and identifier source if these can be deduced from
        the input file name.   Returned values are in uppercase.
        """
        #
        #
        fId = None
        fType = None
        if fileName is None or len(fileName) < 1:
            return fId, fType

        (head, _tail) = os.path.splitext(str(fileName))
        headLC = head.upper()
        if headLC.startswith("RCSB"):
            fId = head
            fType = "RCSB"
        elif headLC.startswith("D_"):
            fId = head
            fType = "WF_ARCHIVE"
            #
            # Look for
            #
            fields = head.split("_")
            if len(fields) > 1:
                fId = "_".join(fields[:2])
        elif headLC.startswith("W_"):
            fId = head
            fType = "WF_INSTANCE"
        else:
            fId = head
            fType = "UNKNOWN"
            if self.__verbose:
                self.__lfh.write("+WebUploadUtils.perceiveIdentifier() using non-standard identifier %r for %r\n" % (head, str(fileName)))

        if self.__verbose:
            self.__lfh.write("+WebUploadUtils.perceiveIdentifier() using identifier fId %r and file source %r \n" % (fId, fType))

        return fId, fType

    def __copyGzip(self, inpFilePath, outFilePath):
        """"""
        try:
            cmd = " gzip -cd  %s > %s " % (inpFilePath, outFilePath)
            os.system(cmd)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            if self.__verbose:
                traceback.print_exc(file=self.__lfh)
            return False



##
# File:  WebDownloadUtils.py
# Date:  22-Mar-2013   J. Westbrook
#
# Updates:
#   06-Mar-2014 jdw -- explicitly set return format in the response object.
#
#
##
"""
Utilities to manage  web application download requests for archive and workflow data files -.

   URL parameters or keys in the request object.

        'data_set_id'
        'file_source'
        'content_type'
        'format'

   optional
          'wf_instance'
          'version'
          'part'
          'compress'

    response corresponds to  -

      for success -
        self.__reqObj.setReturnFormat(return_format="binary")

      and for failure  -

        self.__reqObj.setReturnFormat(return_format="text|html|json")

"""

import sys
import os
from wwpdb.io.locator.PathInfo import PathInfo
from wwpdb.utils.session.WebRequest import ResponseContent

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.09"


class WebDownloadUtils(object):
    """
    This class encapsulates handling download requests for workflow data files -

    """

    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):

        self.__reqObj = reqObj
        self.__verbose = verbose
        self.__lfh = log
        #
        self.__debug = False
        #
        self.__sessionObj = self.__reqObj.getSessionObj()
        self.__sessionPath = self.__sessionObj.getPath()
        self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")

        self.__pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        #
        if self.__verbose:
            self.__lfh.write("+WebDownloadUtils.__setup() - session id   %s\n" % (self.__sessionObj.getId()))
            self.__lfh.write("+WebDownloadUtils.__setup() - session path %s\n" % (self.__sessionPath))

    def makeDownloadResponse(self):
        """Return a response object correponding to a download action for data file described by the
        parameter content in the request object.
        """
        if self.__verbose:
            self.__lfh.write("+WebDownloadUtils.makeResponse() starting with session path %s\n" % self.__sessionPath)
        filePath = self.__getDownloadFileInfo()

        if self.__verbose:
            self.__lfh.write("+WebDownloadUtils.makeResponse() target file path is %s\n" % filePath)
        return self.__makeResponseContentObject(filePath=filePath)

    def __getDownloadFileInfo(self):
        """Extract target file details and return file path or None."""
        retPath = None
        #
        dataSetId = self.__reqObj.getValue("data_set_id")
        if len(dataSetId) < 1:
            return retPath

        fileSource = self.__reqObj.getValueOrDefault("file_source", default="archive")
        if fileSource not in ["archive", "wf-archive", "wf-instance", "session", "wf-session"]:
            return retPath

        wfInstanceId = self.__reqObj.getValueOrDefault("wf_instance", default=None)

        contentType = self.__reqObj.getValue("content_type")
        if len(contentType) < 1:
            return retPath

        formatType = self.__reqObj.getValueOrDefault("format", default="pdbx")
        versionId = self.__reqObj.getValueOrDefault("version", default="latest")
        partNumber = self.__reqObj.getValueOrDefault("part", "1")
        #

        retPath = self.__pI.getFilePath(
            dataSetId, wfInstanceId=wfInstanceId, contentType=contentType, formatType=formatType, fileSource=fileSource, versionId=versionId, partNumber=partNumber
        )
        return retPath

    def __makeResponseContentObject(self, filePath, attachmentFlag=True, compressFlag=False):
        """Create a response content object for the input file"""
        if self.__verbose:
            self.__lfh.write("+WebDownloadUtils.__makeResponseContentObject() starting with file path %s\n" % filePath)
        #
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        if filePath is not None and os.access(filePath, os.F_OK):
            rC.setReturnFormat("binary")
            rC.setBinaryFile(filePath, attachmentFlag=attachmentFlag, serveCompressed=compressFlag)
        else:
            rC.setReturnFormat("json")
            rC.setError(errMsg="Download failure for %s" % filePath)

        return rC



##
# File:  FileUtils.py
# Date:  4-July-2014  J. Westbrook
#
# Update:
#  6-July-2014  jdw add download anchors
#  5- Jan-2015  jdw revise to support access to remote validation server files within the
#                   production context -
# 29- Nov-2016  ep  Add dict-check-report-next to list
# 13- Feb-2016  ep  Add '3DEM Files' to default list for FileUtils.
# 28-Sept-2017  zf  Modified renderFileList() & __renderContentTypeFileList()
##
"""
Manage the presentation of project files for download.


"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import sys
import os.path
import os
import glob

from wwpdb.io.file.DataExchange import DataExchange
from wwpdb.io.locator.PathInfo import PathInfo
from wwpdb.utils.config.ConfigInfo import ConfigInfo


class FileUtilsBase(object):
    """Base class that defines the content types to download"""

    def __init__(self):
        self._rDList = ["Primary Data Files", "Chemical Assignment Files", "Sequence Assignment Files", "Annotation Task Files", "Check reports", "Message Files", "3DEM Files"]
        self._rD = {
            "Primary Data Files": [
                "model",
                "structure-factors",
                "nmr-restraints",
                "nmr-chemical-shifts",
                "nmr-data-str",
                "nmr-data-nef",
                "nmr-peaks",
                "em-volume",
                "em-mask-volume",
                "img-emdb",
            ],
            "Chemical Assignment Files": [
                "chem-comp-link",
                "chem-comp-assign",
                "chem-comp-assign-final",
                "chem-comp-assign-details",
                "chem-comp-depositor-info",
                "prd-search",
                "component-image",
                "component-definition",
                "auxiliary-file",
            ],
            "Sequence Assignment Files": [
                "seqdb-match",
                "blast-match",
                "seq-assign",
                "seq-data-stats",
                "seq-align-data",
                "polymer-linkage-distances",
                "polymer-linkage-report",
                "sequence-fasta",
                "partial-seq-annotate",
            ],
            "Annotation Task Files": [
                "assembly-report",
                "assembly-assign",
                "interface-assign",
                "assembly-model",
                "assembly-model-xyz",
                "site-assign",
                "map-2fofc",
                "map-fofc",
                "secondary-structure-topology",
                "map-header-data",
                "fsc",
            ],
            "Check reports": [
                "validation-report-depositor",
                "geometry-check-report",
                "polymer-linkage-report",
                "dict-check-report",
                "dict-check-report-r4",
                "dict-check-report-next",
                "format-check-report",
                "misc-check-report",
                "special-position-report",
                "merge-xyz-report",
                "nmr-cs-check-report",
                "nmr-cs-xyz-check-report",
                "validation-report",
                "validation-report-full",
                "validation-report-slider",
                "validation-data",
                "validation-report-2fo-map-coef",
                "validation-report-fo-map-coef",
                "sf-convert-report",
                "dcc-report",
                "mapfix-report",
                "fsc-report",
                "em2em-report",
            ],
            "Message Files": ["messages-from-depositor", "messages-to-depositor", "notes-from-annotator", "correspondence-to-depositor"],
            "3DEM Files": [
                "em-volume",
                "em-mask-volume",
                "em-additional-volume",
                "em-half-volume",
                "em-volume-header",
                "em-model-emd",
                "em2em-report",
                "img-emdb",
                "img-emdb-report",
                "layer-lines",
                "auxiliary-file",
            ],
        }


class FileUtils(FileUtilsBase):
    """
    Manage the presentation of project files for download.

    """

    def __init__(self, entryId, reqObj=None, verbose=False, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__reqObj = reqObj
        self.__debug = True
        # Reassign siteId for the following special case --
        self.__entryId = entryId
        siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        # This is for viewing the entries from the standalone validation server from annotation --
        if siteId in ["WWPDB_DEPLOY_PRODUCTION_RU", "WWPDB_DEPLOY_VALSRV_RU", "WWPDB_DEPLOY_TEST", "WWPDB_DEPLOY_INTERNAL_RU"] and entryId.startswith("D_90"):
            siteId = "WWPDB_DEPLOY_VALSRV_RU"
        #
        # Get inventory of file types
        super(FileUtils, self).__init__()
        #
        self.__setup(siteId=siteId)

    def __setup(self, siteId=None):
        if siteId is not None:
            self.__siteId = siteId
        else:
            self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        #
        self.__lfh.write("+FileUtils.__setup() starting with entryId %r adjusted WWPDB_SITE_ID %r\n" % (self.__entryId, self.__siteId))
        #
        self.__sObj = self.__reqObj.getSessionObj()
        self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        self.__pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        self.__cleanup = False
        self.__currentHeaderFilePath = None
        self.__cI = ConfigInfo(self.__siteId)
        self.__cD = self.__cI.get("CONTENT_TYPE_DICTIONARY")
        self.__msL = self.__cI.get("CONTENT_MILESTONE_LIST")
        #

    def renderFileList(self, fileSource="archive", rDList=None, titlePrefix="", titleSuffix="", displayImageFlag=False):
        """"""
        if rDList is None:
            rDList = self._rDList

        htmlList = []
        nTot = 0
        if fileSource in ["archive", "deposit", "wf-archive"]:
            for ky in rDList:
                if ky not in self._rD:
                    continue
                ctList = self._rD[ky]
                title = titlePrefix + ky + titleSuffix
                fList = []
                fList.extend(ctList)
                for ct in ctList:
                    for ms in self.__msL:
                        mt = ct + "-" + ms
                        fList.append(mt)
                nF, oL = self.__renderContentTypeFileList(
                    self.__entryId, fileSource=fileSource, wfInstanceId=None, contentTypeList=fList, title=title, displayImageFlag=displayImageFlag
                )
                if nF > 0:
                    htmlList.extend(oL)
                    nTot += nF

        if fileSource in ["archive", "wf-archive"]:
            nF, oL = self.__renderLogFileList(self.__entryId, fileSource="archive", title="Archive Log Files")
            if nF > 0:
                htmlList.extend(oL)
                nTot += nF

        if fileSource in ["deposit"]:
            nF, oL = self.__renderLogFileList(self.__entryId, fileSource="deposit", title="Deposit Log Files")
            if nF > 0:
                htmlList.extend(oL)
                nTot += nF

        #
        if fileSource in ["wf-instance", "instance"]:
            iTopPath = self.__pI.getInstanceTopPath(self.__entryId)
            fPattern = os.path.join(iTopPath, "*")
            wfInstancePathList = filter(os.path.isdir, glob.glob(fPattern))
            for wfInstancePath in wfInstancePathList:
                (_pth, wfInstId) = os.path.split(wfInstancePath)
                title = "Files in " + wfInstId
                nF, oL = self.__renderWfInstanceFileList(self.__entryId, wfInstancePath, title=title)
                if nF > 0:
                    htmlList.extend(oL)
                    nTot += nF
        #
        return nTot, htmlList

    def __renderContentTypeFileList(self, entryId, fileSource="archive", wfInstanceId=None, contentTypeList=None, title=None, displayImageFlag=False):
        if contentTypeList is None:
            contentTypeList = ["model"]
        if self.__verbose:
            self.__lfh.write(
                "+FileUtils.renderContentTypeFileList() entryId %r fileSource %r wfInstanceId %r contentTypeList %r \n" % (entryId, fileSource, wfInstanceId, contentTypeList)
            )
        de = DataExchange(
            reqObj=self.__reqObj, depDataSetId=entryId, wfInstanceId=wfInstanceId, fileSource=fileSource, siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh
        )
        tupL = de.getContentTypeFileList(fileSource=fileSource, contentTypeList=contentTypeList)
        #
        rTupL = []
        for tup in tupL:
            href, fN = self.__makeDownloadHref(tup[0])
            if tup[2] > 1:
                sz = "%d" % int(tup[2])
            else:
                sz = "%.3f" % tup[2]
            rTup = (href, tup[1], sz)
            rTupL.append(rTup)
            if displayImageFlag and fN.startswith(entryId + "_img-emdb"):
                imgFile = os.path.join(self.__sessionPath, fN)
                if os.access(imgFile, os.F_OK):
                    os.remove(imgFile)
                #
                os.symlink(tup[0], imgFile)
                imgHtml = '<img src="/sessions/' + self.__sessionId + "/" + fN + '" border="0" alt="Image" width="400" height="400">'
                rTupL.append(("displayImage", imgHtml, ""))
            #
        #
        if title is None:
            cS = ",".join(contentTypeList)
            title = "File Source %s (%s)" % (fileSource, cS)
        nF, htmlList = self.__renderFileList(rTupL, title)

        return nF, htmlList

    def __renderWfInstanceFileList(self, entryId, wfPath, title=None):
        if self.__verbose:
            self.__lfh.write("+FileUtils.renderWfInstanceFileList() wfPath %s\n" % wfPath)

        wfPattern = os.path.join(wfPath, "*")
        de = DataExchange(reqObj=self.__reqObj, depDataSetId=entryId, wfInstanceId=None, fileSource=None, siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        tupL = de.getMiscFileList(fPatternList=[wfPattern], sortFlag=True)
        #
        rTupL = []
        for tup in tupL:
            href, _fN = self.__makeDownloadHref(tup[0])
            if tup[2] > 1:
                sz = "%d" % int(tup[2])
            else:
                sz = "%.3f" % tup[2]
            rTup = (href, tup[1], sz)
            rTupL.append(rTup)

        if title is None:
            title = "Workflow instance files for %s" % entryId
        nF, htmlList = self.__renderFileList(rTupL, title)

        return nF, htmlList

    def __renderLogFileList(self, entryId, fileSource="archive", title=None):
        if self.__verbose:
            self.__lfh.write("+FileUtils.renderLogFileList() entryId %r fileSource %r\n" % (entryId, fileSource))
        de = DataExchange(reqObj=self.__reqObj, depDataSetId=entryId, wfInstanceId=None, fileSource=fileSource, siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        tupL = de.getLogFileList(entryId, fileSource=fileSource)
        #
        rTupL = []
        for tup in tupL:
            href, _fN = self.__makeDownloadHref(tup[0])
            if tup[2] > 1:
                sz = "%d" % int(tup[2])
            else:
                sz = "%.3f" % tup[2]
            rTup = (href, tup[1], sz)
            rTupL.append(rTup)

        if title is None:
            title = "Log Files in Source %s" % fileSource
        nF, htmlList = self.__renderFileList(rTupL, title)

        return nF, htmlList

    def __renderFileList(self, fileTupleList, title, embeddedTitle=True):
        #
        oL = []
        if len(fileTupleList) > 0:
            if embeddedTitle:
                oL.append('<table class="table table-bordered table-striped table-condensed">')
                oL.append('<tr><th class="width50">%s</th><th class="width30">Modification Time</th><th class="width20">Size (KBytes)</th></tr>' % title)
            else:
                oL.append("<h4>%s</h4>" % title)
                oL.append('<table class="table table-bordered table-striped table-condensed">')
                oL.append('<tr><th class="width50">Files</th><th class="width30">Modification Time</th><th class="width20">Size (KBytes)</th></tr>')
            for tup in fileTupleList:
                oL.append("<tr>")
                if tup[0] == "displayImage":
                    oL.append('<td align="center" colspan="3">%s</td>' % tup[1])
                else:
                    oL.append("<td>%s</td>" % tup[0])
                    oL.append("<td>%s</td>" % tup[1])
                    oL.append("<td>%s</td>" % tup[2])
                #
                oL.append("</tr>")
            #
            oL.append("</table>")
        #
        return len(fileTupleList), oL

    def __makeDownloadHref(self, filePath):
        _dP, fN = os.path.split(filePath)
        tS = "/service/review_v2/download_file?sessionid=" + self.__sessionId + "&file_path=" + filePath
        href = "<a class='my-file-downloadable' href='" + tS + "'>" + fN + "</a>"
        return href, fN



##
# File:  WebAppWorkerBase.py
# Date:  28-Dec-2013
#
# Updates:
##
"""
Base class for supporting web application processing modules.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import os
import sys
import time
import types
import traceback
import ntpath

#
from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.session.UtilDataStore import UtilDataStore
from wwpdb.utils.session.WebRequest import ResponseContent

#


class WebAppWorkerBase(object):
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        Base class supporting web application worker methods.

        Performs URL -> application mapping for this module.

        """

        self._verbose = verbose
        self.__debug = False
        self._lfh = log
        self._reqObj = reqObj
        self._sObj = None
        self._sessionId = None
        self._sessionPath = None
        self._rltvSessionPath = None
        #
        self._siteId = self._reqObj.getValue("WWPDB_SITE_ID")
        self._cI = ConfigInfo(self._siteId)
        #
        self._uds = None
        # UtilDataStore prefix for general session data -- used by _getSession()
        self._udsPrefix = None
        #
        self.__appPathD = {}

    def addService(self, url, opName):
        self.__appPathD[url] = opName

    def addServices(self, serviceDict):
        for k, v in serviceDict.items():
            self.__appPathD[k] = v

    def doOp(self):
        """Map operation to path and invoke operation.  Exceptions are caught within this method.

        :returns:

        Operation output is packaged in a ResponseContent() object.

        """
        #
        try:
            inpReqPath = self._reqObj.getRequestPath()
            # first pull off the REST style URLS --
            #
            #  /service/review/report/D_XXXXXX
            #
            if inpReqPath.startswith("/service/review/report/d_"):
                rFields = inpReqPath.split("/")
                self._reqObj.setValue("idcode", rFields[4].upper())
                reqPath = "/service/review/report"
            else:
                reqPath = inpReqPath
            #
            if reqPath not in self.__appPathD:
                # bail out if operation is unknown -
                rC = ResponseContent(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
                rC.setError(errMsg="Unknown operation")
            else:
                mth = getattr(self, self.__appPathD[reqPath], None)
                rC = mth()
            return rC
        except:  # noqa: E722 pylint: disable=bare-except
            if self.__debug:
                traceback.print_exc(file=self._lfh)
            rC = ResponseContent(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
            rC.setError(errMsg="Operation failure")
            return rC

    #
    def _saveSessionParameter(self, param=None, value=None, pvD=None, prefix=None):
        """Store the input (param,value) pair and/or the contents of parameter value
        dictionary (pvD) in the session parameter store.
        """
        try:
            # if self._uds is None:
            self._uds = UtilDataStore(reqObj=self._reqObj, prefix=prefix, verbose=self._verbose, log=self._lfh)
            if param is not None:
                self._uds.set(param, value)
                self._uds.serialize()
            if pvD is not None and len(pvD) > 0:
                for k, v in pvD.items():
                    self._uds.set(k, v)
                self._uds.serialize()
            return True
        except Exception as e:
            if self._verbose:
                self._lfh.write("+WebAppWorkerBase._saveSessionParameter() failed in session %s - %r\n" % (self._sessionId, str(e)))
        return False

    def _getSessionParameter(self, param=None, prefix=None):
        """Recover session data for the input parameter or return an empty string."""
        try:
            self._uds = UtilDataStore(reqObj=self._reqObj, prefix=prefix, verbose=self._verbose, log=self._lfh)
            return self._uds.get(param)
        except Exception as e:
            if self._verbose:
                self._lfh.write("+WebAppWorkerBase._getSessionParameter() failed in session %s - %r\n" % (self._sessionId, str(e)))
        return ""

    def _getFileText(self, filePath):
        self._reqObj.setReturnFormat(return_format="text")
        rC = ResponseContent(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
        rC.setTextFile(filePath)
        return rC

    def _newSessionOp(self):
        if self.__debug:
            self._lfh.write("+WebAppWorkerBase.newSessionOp() starting\n")

        self._getSession(forceNew=True)
        self._reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
        sId = self._reqObj.getSessionId()
        if len(sId):
            rC.setHtmlText("Session id %s created." % sId)
        else:
            rC.setError(errMsg="No session created")

        return rC

    def _verifySessionContext(self, apikyfn, overWrite=True):
        try:
            self._sObj = self._reqObj.getSessionObj()
            pth = self._sObj.getPath()
            if pth is None:
                return False
            self._sessionId = self._sObj.getId()
            self._sessionPath = self._sObj.getPath()
            self._rltvSessionPath = self._sObj.getRelativePath()
            uds = UtilDataStore(reqObj=self._reqObj, prefix=self._udsPrefix, verbose=self._verbose, log=self._lfh)
            dd = uds.getDictionary()
            if self.__debug:
                self._lfh.write("+WebAppWorkerBase._verifySessionContext() -  importing persisted general session parameters:\n")
                for k, v in dd.items():
                    if isinstance(v, list) or isinstance(v, dict):
                        self._lfh.write(" %30s length=%d\n" % (k, len(v)))
                    else:
                        self._lfh.write(" %30s= %r\n" % (k, v))
            self._reqObj.setDictionary(dd, overWrite=overWrite)
            #
            # Now check for a valid key --
            #
            reqApiKey = self._reqObj.getValue("apikey")
            ok = apikyfn(reqApiKey)
            return ok
        except Exception as e:
            if self._verbose:
                self._lfh.write("+WebAppWorkerBase._verifySessionContext() - failed - %r\n" % str(e))
            if self._verbose:
                traceback.print_exc(file=self._lfh)
        return False

    def _getSession(self, forceNew=False, useContext=False, overWrite=True):
        """Join existing session or create new session as required."""
        #
        self._sObj = self._reqObj.newSessionObj(forceNew=forceNew)

        self._sessionId = self._sObj.getId()
        self._sessionPath = self._sObj.getPath()
        self._rltvSessionPath = self._sObj.getRelativePath()

        if self._verbose:
            self._lfh.write("+WebAppWorkerBase._getSession() - session   id  %s\n" % self._sessionId)
            self._lfh.write("+WebAppWorkerBase._getSession() - session path  %s\n" % self._sessionPath)

        if useContext:
            uds = UtilDataStore(reqObj=self._reqObj, prefix=self._udsPrefix, verbose=self._verbose, log=self._lfh)
            dd = uds.getDictionary()
            if self.__debug:
                self._lfh.write("+WebAppWorkerBase._getSession() -  importing persisted general session parameters:\n")
                for k, v in dd.items():
                    if isinstance(v, list) or isinstance(v, dict):
                        self._lfh.write(" %30s length=%d\n" % (k, len(v)))
                    else:
                        self._lfh.write(" %30s= %r\n" % (k, v))
            self._reqObj.setDictionary(dd, overWrite=overWrite)

    def _isFileUpload(self, fileTag="file"):
        """Generic check for the existence of request paramenter of type "file"."""
        fs = self._reqObj.getRawValue(fileTag)
        if sys.version_info[0] < 3:
            if (fs is None) or (isinstance(fs, types.StringType)):  # pylint: disable=no-member
                return False
        else:
            if (fs is None) or (isinstance(fs, str)) or (isinstance(fs, bytes)):
                return False

        return True

    def _uploadFile(self, fileTag="file"):
        """Copying uploaded file to the session directory.  Return file name or None."""
        try:
            fs = self._reqObj.getRawValue(fileTag)
            fNameInput = str(fs.filename)

            #
            # Need to deal with some platform issues -
            #
            if fNameInput.find("\\") != -1:
                # likely windows path -
                fName = ntpath.basename(fNameInput)
            else:
                fName = os.path.basename(fNameInput)

            #
            # Store upload file in session directory -
            #
            fPathAbs = os.path.join(self._sessionPath, fName)
            if self._verbose:
                self._lfh.write("+WebAppWorkerBase._uploadFile() - starting upload of %r to path %r\n" % (fNameInput, fPathAbs))

            ofh = open(fPathAbs, "wb")
            ofh.write(fs.file.read())
            ofh.close()

            #
            if self._verbose:
                self._lfh.write("+WebAppWorkerBase._uploadFile() - uploaded completed for file tag %s file name %s\n" % (fileTag, fName))
            #
            #  Store the file path and name in request object -
            #
            self._reqObj.setValue("filePath", fPathAbs)
            self._reqObj.setValue("fileName", fName)
            return fName
        except Exception as e:
            if self._verbose:
                self._lfh.write("+WebAppWorkerBase._uploadFile() - Upload failed for file tag %s file name %s - %r\n" % (fileTag, fs.filename, str(e)))
            if self.__debug:
                traceback.print_exc(file=self._lfh)
        return None

    def _setSemaphore(self):
        sVal = str(time.strftime("TMP_%Y%m%d%H%M%S", time.localtime()))
        self._reqObj.setValue("semaphore", sVal)
        return sVal

    def _openSemaphoreLog(self, semaphore="TMP_"):
        sessionId = self._reqObj.getSessionId()
        sessionPath = self._reqObj.getSessionPath()
        fPathAbs = os.path.join(sessionPath, sessionId, semaphore + ".log")
        self._lfh = open(fPathAbs, "w")

    def _closeSemaphoreLog(self, semaphore="TMP_"):  # pylint: disable=unused-argument
        self._lfh.flush()
        self._lfh.close()

    def _postSemaphore(self, semaphore="TMP_", value="OK"):
        sessionId = self._reqObj.getSessionId()
        sessionPath = self._reqObj.getSessionPath()
        fPathAbs = os.path.join(sessionPath, sessionId, semaphore)
        fp = open(fPathAbs, "w")
        fp.write("%s\n" % value)
        fp.close()
        return semaphore

    def _semaphoreExists(self, semaphore="TMP_"):
        sessionId = self._reqObj.getSessionId()
        sessionPath = self._reqObj.getSessionPath()
        fPathAbs = os.path.join(sessionPath, sessionId, semaphore)
        if os.access(fPathAbs, os.F_OK):
            return True
        else:
            return False

    def _getSemaphore(self, semaphore="TMP_"):
        sessionId = self._reqObj.getSessionId()
        sessionPath = self._reqObj.getSessionPath()
        fPathAbs = os.path.join(sessionPath, sessionId, semaphore)
        if self._verbose:
            self._lfh.write("+ReviewDataWebApp.__getSemaphore() - checking %s in path %s\n" % (semaphore, fPathAbs))
        try:
            fp = open(fPathAbs, "r")
            lines = fp.readlines()
            fp.close()
            sval = lines[0][:-1]
        except:  # noqa: E722 pylint: disable=bare-except
            sval = "FAIL"
        return sval
